@@ -18,10 +18,12 @@ import cfnlint
 from cfnlint.decode.node import str_node, dict_node, list_node, sub_node
 
 try:
-    from yaml.cyaml import CParser as Parser  # pylint: disable=ungrouped-imports
+    from yaml._yaml import CParser as Parser  # pylint: disable=ungrouped-imports,
+
     cyaml = True
 except ImportError:
-    from yaml.parser import Parser  # pylint: disable=ungrouped-imports
+    from yaml.parser import Parser  # type: ignore # pylint: disable=ungrouped-imports
+
     cyaml = False
 
 UNCONVERTED_SUFFIXES = ['Ref', 'Condition']
@@ -41,16 +43,24 @@ class CfnParseError(ConstructorError):
             errors = [errors]
 
         # Call the base class constructor with the parameters it needs
-        super(CfnParseError, self).__init__(errors[0].message)
+        super().__init__(errors[0].message)
 
         # Now for your custom code...
         self.filename = filename
         self.matches = errors
 
+
 def build_match(filename, message, line_number, column_number, key):
     return cfnlint.rules.Match(
-        line_number + 1, column_number + 1, line_number + 1,
-        column_number + 1 + len(key), filename, cfnlint.rules.ParseError(), message=message)
+        line_number + 1,
+        column_number + 1,
+        line_number + 1,
+        column_number + 1 + len(key),
+        filename,
+        cfnlint.rules.ParseError(),
+        message=message,
+    )
+
 
 class NodeConstructor(SafeConstructor):
     """
@@ -59,7 +69,7 @@ class NodeConstructor(SafeConstructor):
 
     def __init__(self, filename):
         # Call the base class constructor
-        super(NodeConstructor, self).__init__()
+        super().__init__()
 
         self.filename = filename
 
@@ -75,50 +85,65 @@ class NodeConstructor(SafeConstructor):
         # occurance, which can give unexpected results
         mapping = {}
         self.flatten_mapping(node)
+        matches = []
         for key_node, value_node in node.value:
             key = self.construct_object(key_node, False)
             value = self.construct_object(value_node, False)
 
             for key_dup in mapping:
                 if key_dup == key:
-                    raise CfnParseError(
-                        self.filename,
-                        [
+                    if not matches:
+                        matches.extend(
+                            [
+                                build_match(
+                                    filename=self.filename,
+                                    message=f'Duplicate found "{key}" (line {key_dup.start_mark.line + 1})',
+                                    line_number=key_dup.start_mark.line,
+                                    column_number=key_dup.start_mark.column,
+                                    key=key,
+                                ),
+                                build_match(
+                                    filename=self.filename,
+                                    message=f'Duplicate found "{key}" (line {key_node.start_mark.line + 1})',
+                                    line_number=key_node.start_mark.line,
+                                    column_number=key_node.start_mark.column,
+                                    key=key,
+                                ),
+                            ],
+                        )
+                    else:
+                        matches.append(
                             build_match(
                                 filename=self.filename,
-                                message='Duplicate resource found "{}" (line {})'.format(
-                                    key, key_dup.start_mark.line + 1),
-                                line_number=key_dup.start_mark.line,
-                                column_number=key_dup.start_mark.column,
-                                key=key
-                            ),
-                            build_match(
-                                filename=self.filename,
-                                message='Duplicate resource found "{}" (line {})'.format(
-                                    key, key_node.start_mark.line + 1),
+                                message=f'Duplicate found "{key}" (line {key_node.start_mark.line + 1})',
                                 line_number=key_node.start_mark.line,
                                 column_number=key_node.start_mark.column,
-                                key=key
+                                key=key,
                             ),
-                        ]
-                    )
+                        )
             try:
                 mapping[key] = value
-            except:
+            except Exception as exc:
                 raise CfnParseError(
-                        self.filename,
-                        [
-                            build_match(
-                                filename=self.filename,
-                                message=f'Unhashable type "{key}" (line {key.start_mark.line + 1})',
-                                line_number=key.start_mark.line,
-                                column_number=key.start_mark.column,
-                                key=key
-                            ),
-                        ]
-                    )
+                    self.filename,
+                    [
+                        build_match(
+                            filename=self.filename,
+                            message=f'Unhashable type "{key}" (line {key.start_mark.line + 1})',
+                            line_number=key.start_mark.line,
+                            column_number=key.start_mark.column,
+                            key=key,
+                        ),
+                    ],
+                ) from exc
 
-        obj, = SafeConstructor.construct_yaml_map(self, node)
+        if matches:
+            raise CfnParseError(
+                self.filename,
+                matches,
+            )
+
+        (obj,) = SafeConstructor.construct_yaml_map(self, node)
 
         if len(mapping) == 1:
             if 'Fn::Sub' in mapping:
@@ -132,28 +157,29 @@ class NodeConstructor(SafeConstructor):
         return str_node(obj, node.start_mark, node.end_mark)
 
     def construct_yaml_seq(self, node):
-        obj, = SafeConstructor.construct_yaml_seq(self, node)
+        (obj,) = SafeConstructor.construct_yaml_seq(self, node)
         assert isinstance(obj, list)
         return list_node(obj, node.start_mark, node.end_mark)
 
 
-NodeConstructor.add_constructor(
-    u'tag:yaml.org,2002:map',
-    NodeConstructor.construct_yaml_map)
+NodeConstructor.add_constructor(  # type: ignore
+    'tag:yaml.org,2002:map', NodeConstructor.construct_yaml_map
+)
 
-NodeConstructor.add_constructor(
-    u'tag:yaml.org,2002:str',
-    NodeConstructor.construct_yaml_str)
-
-NodeConstructor.add_constructor(
-    u'tag:yaml.org,2002:seq',
-    NodeConstructor.construct_yaml_seq)
+NodeConstructor.add_constructor(  # type: ignore
+    'tag:yaml.org,2002:str', NodeConstructor.construct_yaml_str
+)
 
+NodeConstructor.add_constructor(  # type: ignore
+    'tag:yaml.org,2002:seq', NodeConstructor.construct_yaml_seq
+)
 
+# pylint: disable=too-many-ancestors
 class MarkedLoader(Reader, Scanner, Parser, Composer, NodeConstructor, Resolver):
     """
     Class for marked loading YAML
     """
+
     # pylint: disable=non-parent-init-called,super-init-not-called
 
     def __init__(self, stream, filename):
@@ -175,7 +201,7 @@ def multi_constructor(loader, tag_suffix, node):
     """
 
     if tag_suffix not in UNCONVERTED_SUFFIXES:
-        tag_suffix = '{}{}'.format(FN_PREFIX, tag_suffix)
+        tag_suffix = f'{FN_PREFIX}{tag_suffix}'
 
     constructor = None
     if tag_suffix == 'Fn::GetAtt':
@@ -187,7 +213,7 @@ def multi_constructor(loader, tag_suffix, node):
     elif isinstance(node, MappingNode):
         constructor = loader.construct_mapping
     else:
-        raise 'Bad tag: !{}'.format(tag_suffix)
+        raise f'Bad tag: !{tag_suffix}'
 
     if tag_suffix == 'Fn::Sub':
         return sub_node({tag_suffix: constructor(node)}, node.start_mark, node.end_mark)
@@ -205,7 +231,7 @@ def construct_getatt(node):
     if isinstance(node.value, list):
         return list_node([s.value for s in node.value], node.start_mark, node.end_mark)
 
-    raise ValueError('Unexpected node type: {}'.format(type(node.value)))
+    raise ValueError(f'Unexpected node type: {type(node.value)}')
 
 
 def loads(yaml_string, fname=None):
@@ -233,7 +259,7 @@ def load(filename):
         for line in fileinput.input(files=filename):
             content = content + line
     else:
-        with open(filename) as fp:
+        with open(filename, encoding='utf-8') as fp:
             content = fp.read()
 
     return loads(content, filename)
